package sim;

import java.util.Iterator;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.Queue;
import java.util.Random;

import sim.Event.EventType;
import bean.Reservation;
import bean.Reservation.VehicleClass;

public class Simulatore {

	private Queue<Event> eventQueue;
	private int[] availableCars;
	private int[] freeUpgrades;
	private int disasters;
	private int totRented;
	Map<Integer, Reservation> reservations;
	
	
	public Simulatore(Map<Integer, Reservation> reservations) {
		this.reservations = reservations;
		disasters = 0;
		totRented = 0;
		eventQueue = new PriorityQueue<Event>();
		availableCars = new int[VehicleClass.values().length];
		freeUpgrades = new int[VehicleClass.values().length];
		
		for(int i=0; i<VehicleClass.values().length; ++i) {
			availableCars[i] =0;
			freeUpgrades[i] =0;
		}
		
		Iterator<Reservation> it = reservations.values().iterator();
		while (it.hasNext()) {
			Reservation r = it.next();
			Random rand = new Random();
			
			eventQueue.add(new Event(r.getPickupDay(), EventType.CAR_PICKUP, r.getId()));
			eventQueue.add(new Event(r.getReturnDay() + rand.nextInt(5), EventType.CAR_RETURN, r.getId()));
		}
	}
	
	public void setAvailability(int type, int num) {
		availableCars[type] = num;
	}
	
	public boolean moreEvents() {
		return !eventQueue.isEmpty();
	}
	
	public void print() {
		System.out.println("Affitate " + totRented + " auto");
		
		System.out.println("Disastri: " + disasters);
		
		for (int i=1; i<VehicleClass.values().length; ++i) {
			System.out.println("free Upgrade: " + freeUpgrades[i] + " classe: " + i);
		}
	}
	
	public void step() {
		Event e = eventQueue.remove();
		Reservation r = reservations.get(e.reservationID);
		
		switch (e.type) {
		case CAR_PICKUP:
			++totRented;
			if (availableCars[r.getVehicleRequested().ordinal()] > 0){
				// auto richiesta presente.
				r.setVehicleRented(r.getVehicleRequested());
				--availableCars[r.getVehicleRequested().ordinal()];
			} else {
				int req = r.getVehicleRequested().ordinal();
				int ottenuta = req;
				
				while(ottenuta > 0 && availableCars[ottenuta] == 0) {
					--ottenuta;
				}
				if (ottenuta == 0) {
					//disastro
					++disasters;
					r.setVehicleRented(r.getVehicleRequested());
				}
				else {
					//freeupgrades
					++freeUpgrades[req];
					r.setVehicleRented(VehicleClass.values()[ottenuta]);
					--availableCars[r.getVehicleRented().ordinal()];
				}
				
				System.out.println("richiesta: " + r.getVehicleRequested() + ", ottenuta: " + r.getVehicleRented());
				
			}
			
			break;
			
		case CAR_RETURN:
			++availableCars[r.getVehicleRented().ordinal()];
			break;
			
		default:
			throw new IllegalArgumentException("Evento sconosciuto.");
		}
	}
}
